	LIST p=16f628a
	; Include file, change directory if needed
	include "P16f628a.inc"

__CONFIG _XT_OSC & _LVP_OFF & _MCLRE_OFF & _PWRTE_ON & _CP_OFF & _DATA_CP_OFF & _WDT_OFF
;_LVP_OFF    RB4
;_XT_OSC   : 4, 2x33

;-- -----------------------------------------------
;--  PORTS settings
;-- -----------------------------------------------
	#define DQ1 PORTA, 0
	#define DQ2 PORTA, 1

	#define DQ1_IO TRISA, 0
	#define DQ2_IO TRISA, 1


;-- MACROS
init_p macro parameter1, parameter2
	movfw parameter2			;-- give W=parameter2
	movwf parameter1			;-- give parameter1=W
	endm

init_v macro parameter, value		;-- macro used to init a variable/register			
	movlw value				;-- give W=value
	movwf parameter				;-- give parameter=W
	endm


	;CBLOCK H'20'	;-- start to declare variables at address H'20'
	;ENDC

;    ,  
dat0        UDATA
;--  1 WIRE
W1_Bus 		RES		1
W1_Bit 		RES		1
_PRESENCE_PULSE 		RES		1
;--  TEMPERATURE DISPLAY
t_lsb 		RES		1
t	 		RES		1
;-- variables for interrupt handler, dalay procedures
w_temp 		RES		1 ; variable used for context saving
status_temp RES		1 ; variable used for context saving
DELAY_CNT	RES		1 ; must be initialized before CALL Delay
CntL		RES		1
CntH		RES		1


.reset		CODE 0x000
		GOTO	Start

	; Start application beyond vector area

.interrupt	CODE 0x004
		GOTO	ISR
	ISR
		MOVWF	w_temp 			; save W and STATUS
		MOVF 	STATUS,W
		MOVWF 	status_temp
		; isr code goes here
		; isr code goes here
		MOVF	status_temp,W	; restore pre-isr STATUS register
		MOVWF	STATUS
		SWAPF	w_temp,F		; restore pre-isr W register contents
		SWAPF	w_temp,W		; ... without affecting the zero flag
	RETFIE 						; return from interrupt

.program	CODE

; ===========================================
; -
; ===========================================
;   4*(DELAY_CNT+1)   5*(DELAY_CNT+1)  (  4 1   1 )
Delay
		NOP					; 1
		DECFSZ	DELAY_CNT,1	; 1(2) 	
		GOTO	Delay		; 2
	RETURN					; 2

;-- ------------------------------------------------------
;--  Project: 	DIGITAL CLOCK, DATE, THERMOMETER 
;-- 				WITH PIC16F84(A)/PIC16F628(A)/PIC16F690 AND DS18S20/DS18B20
;--				+ (in the future: ALARM, THERMOSTAT with DS1631 but only for F690)
;--  Electric diagram & Software: Bogdan Borca/www.bogdi.ro, Timisoara, RO
;--  -----------------------------------------------------

	;-- -----------------------------------------------
	;--  THERMOMETER DS18SB20 OPERATIONS 1 WIRE
	;-- -----------------------------------------------

DQ1_Input ;MACRO
		BSF	STATUS, RP0	; select BANK 1
		BSF TRISA, 0	; Make PORTA[0] an input
		BCF	STATUS, RP0	; select BANK 0
	RETURN	;       
	;ENDM

DQ1_Output ;MACRO
		BSF STATUS, RP0
		BCF TRISA, 0	; Make PORTA[0] an output
		BCF	STATUS, RP0	
	RETURN	;       
	;ENDM

;DQ2_Input
;		BSF STATUS, RP0	; select BANK 1
;		BSF TRISA, 1	; Make PORTA[1] an input
;		BCF	STATUS, RP0	; select BANK 0
;	RETURN

;DQ2_Output
;		BSF STATUS, RP0
;		BCF TRISA, 1	; Make PORTA[1] an output
;		BCF	STATUS, RP0	
;	RETURN

;------------------------------------------------------------------------
; Quotation from DS18B20 datasheet:
;The initialization sequence consists of a reset pulse transmitted by 
;the bus master followed by presence pulse(s) transmitted by the slave(s).
;------------------------------------------------------------------------

	;-- -----------------------------------------------
	;--  RESET PULSE : The master generates RESET PULSE by 
	;--  pulling DQ low from 480us to 960us
	;-- -----------------------------------------------


RESET_PULSE	;RESET_PULSE1
	; RESET_PULSE is sent by master 
	; I have to wait with DQ low for about 480us
		CALL	DQ1_Output
		BCF		DQ1
		init_v	DELAY_CNT,D'120'
		CALL	Delay				; 120*4(5)=480(600)
		CALL	DQ1_Input
		init_v	DELAY_CNT,D'4'
		CALL	Delay				; 4*4(5)=16(20)
	RETURN

;RESET_PULSE2
;DQ2_Output
;DQ2
;etc.
;	RETURN

	;-- -----------------------------------------------
	;--  PRESENCE PULSE : The DS18S20 generates PRESENCE PULSE 
	;--  pulling DQ low between 60 and 240 us after 15us after RESET PULSE
	;-- -----------------------------------------------

PRESENCE_PULSE	;PRESENCE_PULSE1
		CLRF 	_PRESENCE_PULSE
		init_v	DELAY_CNT,D'10'
		CALL	Delay				; 10*4(5) = 40(50) wait and then read presence pulse that lasts 60..240us
		BTFSS 	DQ1					; After more than 60 us and less than 240us read RA0, it should be 0
		BSF 	_PRESENCE_PULSE, 0	; if(DQ1==0) _PRESENCE_PULSE=1, i.e. DS18S20 responded by pulling DQ low
									; Complete the PRESENCE PULSE by waiting to 480 us
		init_v	DELAY_CNT,D'105'
		CALL	Delay				; 420
		BTFSC 	DQ1					; After more than 60 us and less than 240us read RA0, it should be 0
		BSF 	_PRESENCE_PULSE, 1	; if(DQ1==1) _PRESENCE_PULSE+=2, i.e. DS18S20 responded by pulling DQ low

;wait_after_presence_pulse	
;	btfsc DQ			; problematic if lost power during this operation might stuck the clock
		RETURN
;	goto wait_after_presence_pulse


	;-- -----------------------------------------------
	;--  TRANSMIT 1 on the DQ line
	;--  Generate write time slot
	;-- -----------------------------------------------

W1_tx_bit_1
		CALL 	DQ1_Output
		BCF 	DQ1					; DQ=0 for about 9us
		NOP
		NOP
		NOP	
		NOP
		NOP
		CALL 	DQ1_Input
		init_v	DELAY_CNT,D'15'
		CALL	Delay
		GOTO	continue_tx

	;-- -----------------------------------------------
	;--  TRANSMIT 0 on the DQ line
	;--  Generate write time slot
	;-- -----------------------------------------------

W1_tx_bit_0
		CALL	DQ1_Output
		BCF		DQ1					; DQ=0 for about
		init_v	DELAY_CNT,D'15'
		CALL	Delay
		CALL	DQ1_Input
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP					; 6us to let the resistor pulls DQ up
		GOTO	continue_tx

	;-- -----------------------------------------------
	;--  TRANSMIT BYTE
	;--  Initialize W1_Bus with what needs to be send and
	;--  call W1_tx_byte
	;-- -----------------------------------------------

SEND macro what
	init_v W1_Bus, what
	call W1_tx_byte
	endm

	;-- -----------------------------------------------
	;--  TRANSMIT BYTE: The master transmits W1_Bus on the DQ line 
	;-- -----------------------------------------------

W1_tx_byte
	init_v W1_Bit, H'8'
tx	
	btfss W1_Bus, 0
		goto W1_tx_bit_0
		goto W1_tx_bit_1
continue_tx
	rrf W1_Bus, 1			; Right shift the bus
	btfss STATUS, C
	bcf W1_Bus, 7
	decfsz W1_Bit, 1
	goto tx
	return

	;-- -----------------------------------------------
	;--  RECEIVE BYTE: The master receives W1_Data LSB then MSB on the DQ line 
	;-- -----------------------------------------------

W1_Rx_byte	
		init_v W1_Bit, H'8'
RX
;		call test_INT 			; Test if interrupt approach
		bsf STATUS, RP0
		bcf DQ1_IO				;-- Make DQ output
		bcf STATUS, RP0

		bcf DQ1					; Pull low for 5 us	
		nop
		nop
		nop

		bsf STATUS, RP0
		bsf DQ1_IO				; Make DQ an input
		bcf	STATUS, RP0	

		nop
		nop
		nop
		nop						; Wait 9us and then READ
		btfss DQ1				; Test bit that should come from DS18S20
		goto clear_b7
		bsf W1_Bus, 7		; If the bit is set (1) then set the Bus Bit
		goto skip_clear_b7
clear_b7
		bcf W1_Bus, 7			; If the bit is clear (0) then clear Bus Bit	
skip_clear_b7
		init_v	DELAY_CNT,D'12'
		CALL	Delay			; Wait 48 us to complete the read slot
		decfsz W1_Bit, 0		; I do not want any shifts after the last bit was received
		goto rrf__
	return
rrf__
		rrf W1_Bus, 1			; Right shift the bus
		btfss STATUS, C			; If carry is 1 
		bcf W1_Bus, 7			; then clear it

		decf W1_Bit, 1
		goto RX	



	;-- -----------------------------------------------
	;--  Convert & Read can't be made in the same time
	;--  because conversion takes 750ms, so I'll convert
	;--  when date is shown (5 seconds earlier) and
	;--  get what was converted when temperature is shown
	;--  (5 seconds later)
	;-- -----------------------------------------------

sensor_convert
								;-- start a conversion, I don't want multiple conversions and not gets for the temperature
		CALL RESET_PULSE		;-- START/RESET PULSE
		CALL PRESENCE_PULSE		;-- PRESENCE PULSE
		SEND H'CC'				;-- Send skip ROM command
		SEND H'44'				;-- Send convert T command	
	RETURN

sensor_read
	clrf t
	call RESET_PULSE			;-- START/RESET PULSE
	call PRESENCE_PULSE			;-- PRESENCE PULSE
	btfss _PRESENCE_PULSE, 0	;-- If no _PRESENCE_PULSE (_PRESENCE_PULSE[0] = 0) then exit
		RETURN	
								;-- If _PRESENCE_PULSE[0] = 1 then goto back_.. will be skipped
	SEND H'CC'					;-- Send skip ROM command
	SEND H'BE'					;-- Send read Scratchpad command
	call W1_Rx_byte				;-- GET LSB of the temperature in t_lsb
	init_p t_lsb, W1_Bus

	;begin debug purpose only
	MOVWF	w_temp
	MOVF	t_lsb,0
	MOVWF	PORTB				;      PORTB      DS18B20
	MOVF	w_temp,0
	;end debug purpose only
	
	call W1_Rx_byte				;-- GET MSB of the temperature in t, this is in fact 
	init_p t, W1_Bus			;-- the sign written in the whole register

	;begin debug purpose only
	MOVWF	w_temp
	MOVF	t,0
	MOVWF	PORTB				;      PORTB      DS18B20
	MOVF	w_temp,0
	;end debug purpose only

	call RESET_PULSE			;-- START/RESET PULSE, don't care the rest of 7 bytes
	
;-- DS18S20 code
;	#ifdef DS18S20
;	rrf t_lsb, 1				;-- I don't care about the last bit because it
;								;-- represents 2^-1 so I right shift the register
;	#endif						;-- and drop it
	
;-- DS18B20 code
;	#ifdef DS18B20
	rrf t_lsb, 1				;-- I don't care about the last 4 bits because 
	rrf t_lsb, 1				;-- they are 2^-4 2^-3 2^-2 2^-1
	rrf t_lsb, 1
	rrf t_lsb, 1				;-- So, I right shift the register and drop them
							
	bsf t_lsb, 4
	bsf t_lsb, 5
	bsf t_lsb, 6
	btfss t, 0
	bcf t_lsb, 4
	btfss t, 1
	bcf t_lsb, 5
	btfss t, 2
	bcf t_lsb, 6	
;	#endif	

PUT_SIGN
	bsf t_lsb, 7				;-- In t I have the sign 
	btfss t, 4					;-- Test if t is 0 or 1 
		bcf t_lsb, 7			;-- If t[4] is 1 this will be skipped, else t_lsb[7]=0
	init_p t, t_lsb
	RETURN	; sensor_read
; -------------------------------------------------
; end of INC-file
; -------------------------------------------------

GetTemperature1
		CALL	sensor_convert

		; BigDelay for more then 750ms
		MOVLW	D'3'
		MOVWF	CntH
	LoopH
		MOVLW	D'255'
		MOVWF	CntL
	LoopL
		NOP
		DECFSZ	CntL,1
		GOTO	LoopL
		DECFSZ	CntH,1
		GOTO	LoopH

		CALL	sensor_read

		; BigDelay2 for more then 750ms
		MOVLW	D'3'
		MOVWF	CntH
	LoopH2
		MOVLW	D'255'
		MOVWF	CntL
	LoopL2
		NOP
		DECFSZ	CntL,1
		GOTO	LoopL2
		DECFSZ	CntH,1
		GOTO	LoopH2
	RETURN


; ===========================================
;    ()
; ===========================================
; ()      ,  ,  , ,   
Start
		;config registers for input/output
		;CLRF    PORTA			;      
		BSF		STATUS, RP0			; select bank 1
		CLRF 	TRISB				; Make PORTB all outputs
		init_v	TRISA, B'00000011'	; Make PORTA outputs/inputs
		BCF		STATUS, RP0			; back to bank 0
		CLRF	PORTB		;   
							; CMCON setup
		MOVLW  	0x07		; Turn Comparator off and
		MOVWF 	CMCON		; Enable pins for I/O
main_loop
		CALL	GetTemperature1		
		;CALL	GetTemperature2
		;CALL	LCD_Init
		;CALL	ShowTemperatures
		GOTO	main_loop
	END